package es.ua.dlsi.prog3.exceptions;

import java.io.File;
/**
 * Utility class used for managing files and directories
 * @author David Rizo
 */
public class FileUtilities {
	/**
	 * Open a file in the given directory
	 * @param folderName Name of the directory
	 * @param fileName Name of the file 
	 * @param forWriting If true the result file must have write permissions 
	 * @return The actual file object, null if an error has been found, i.e.: folderName does not exist
	 * or it is not a directory, or fileName is not found or it is not a file, or when forWriting is true
	 * the file is not writable. The way of returning null when an error is found is a "traditional" method to
	 * communicate error situations. In any of these cases a message through the standard error flow is printed.  
	 */
	public File getFileInDirectory(String folderName, String fileName, boolean forWriting) {
		File result = null;
		File folder = new File(folderName);
		
		// beginning of spaguetti code for error managing
		if (!folder.exists()) {
			System.err.println("The folder '"+ folderName + "' does not exist");
		} else if (!folder.isDirectory()) {
			System.err.println("'" + folderName + "' is not a folder"); 
		} else {
			File file = new File(folder, fileName);
			if (!forWriting && !file.exists()) {
				System.err.println("The file '" +fileName+ "' does not exist");
			} else if ((!forWriting || forWriting && file.exists()) && !file.isFile()) {
				System.err.println("'" + folderName + "' is not a file");
			} else if (forWriting && file.exists() && !file.canWrite()) {
				System.err.println("'" + fileName + "' has not write permissions");
			} else {
				result = file; 
			}
		}	
		// end of spaguetti code  for error managing
		return result;		
	}
}
