package es.ua.dlsi.prog3.exceptions;

import java.io.File;
/**
 * @author David Rizo
 */
public class FileUtilitiesV2 {
	public static enum Errors { none, eFolderNotExists, eNotAFolder, eFileNotExists, eNotAFile, eCannotWrite};
	public static Errors lastError;
	/**
	 * Open a file in the given directory
	 * @param folderName Name of the directory
	 * @param fileName Name of the file 
	 * @param forWriting If true the result file must have write permissions 
	 * @return The actual file object, null if an error has been found, i.e.: folderName does not exist
	 * or it is not a directory, or fileName is not found or it is not a file, or when forWriting is true
	 * the file is not writable. The way of returning null when an error is found is a "traditional" method to
	 * communicate error situations.   
	 */
	public File getFileInDirectory(String folderName, String fileName, boolean forWriting) {
		lastError = Errors.none;
		
		File result = null;
		File folder = new File(folderName);
		
		// beginning of spaguetti code for error managing
		if (!folder.exists()) {
			lastError = Errors.eFolderNotExists;
		} else if (!folder.isDirectory()) {
			lastError = Errors.eNotAFolder; 
		} else {
			File file = new File(folder, fileName);
			if (!forWriting && !file.exists()) {
				lastError = Errors.eFileNotExists;
			} else if ((!forWriting || forWriting && file.exists()) && !file.isFile()) {
				lastError = Errors.eNotAFile;
			} else if (forWriting && file.exists() && !file.canWrite()) {
				lastError = Errors.eCannotWrite;
			} else {
				result = file; 				
			}
		}	
		// end of spaguetti code  for error managing
		return result;		
	}
	
	/**
	 * Create a File descriptor for a directorio
	 * @param folderName Name of the directory
	 * @return The actual file object, null if an error has been found, i.e.: folderName does not exist
	 * or it is not a directory. The way of returning null when an error is found is a "traditional" method to
	 * communicate error situations.   
	 */
	public File getDirectory(String folderName) {
		lastError = Errors.none;
		
		File result = null;
		File folder = new File(folderName);
		
		// beginning of spaguetti code for error managing
		if (!folder.exists()) {
			lastError = Errors.eFolderNotExists;
		} else if (!folder.isDirectory()) {
			lastError = Errors.eNotAFolder; 
		} else {
			result = folder;
		}	
		// end of spaguetti code  for error managing
		return result;		
	}	
	
	public Errors getLastError() { return lastError; }
	
}
