package es.ua.dlsi.prog3.exceptions;

import java.io.File;
import java.io.FileNotFoundException;
/**
 * @author David Rizo
 */
public class FileUtilitiesV3 {
	/**
	 * Open a file in the given directory
	 * @param folderName Name of the directory
	 * @param fileName Name of the file 
	 * @param forWriting If true the result file must have write permissions 
	 * @return The actual file object, null if an error has been found, i.e.: folderName does not exist
	 * or it is not a directory, or fileName is not found or it is not a file, or when forWriting is true
	 * the file is not writable.    
	 * @throws Exception 
	 */
	public File getFileInDirectory(String folderName, String fileName, boolean forWriting) throws Exception  {
		File folder = new File(folderName);
		
		if (!folder.exists()) {
			throw new FileNotFoundException("The folder '"+ folderName + "' does not exist");
		} if (!folder.isDirectory()) {
			throw new Exception("'" + folderName + "' is not a folder"); 
		} 
			File file = new File(folder, fileName);
			
			/**
			 * Sequence of checks not mixed with business logic code
			 */
			if (!forWriting && !file.exists()) {
				throw new FileNotFoundException("The file '" +fileName+ "' does not exist");
			} 
			
			if ((!forWriting || forWriting && file.exists()) && !file.isFile()) {
				throw new Exception("'" + fileName + "' is not a file");
			} 
			
			if (forWriting && !file.exists() && !file.canWrite()) {
				throw new Exception("'" + fileName + "' has not write permissions");
			} 
			
			return file;
		}	
	}

