package es.ua.dlsi.prog3.exceptions;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
/**
 * @author David Rizo
 */
public class FileUtilitiesV7 {
	/**
	 * Open a file in the given directory
	 * @param folderName Name of the directory
	 * @param fileName Name of the file 
	 * @param forWriting If true the result file must have write permissions 
	 * @return The actual file object, null if an error has been found, i.e.: folderName does not exist
	 * or it is not a directory, or fileName is not found or it is not a file, or when forWriting is true
	 * the file is not writable.    
	 * @throws FileNotFoundException 
	 * @throws NotAFolderException 
	 * @throws NotAFileException 
	 * @throws FileNotWriteableException 
	 * @throws Exception 
	 */
	public File getFileInDirectory(String folderName, String fileName, boolean forWriting) throws FileNotFoundException, NotAFolderException, NotAFileException, FileNotWriteableException {
		File folder = new File(folderName);
		
		if (!folder.exists()) {
			throw new FileNotFoundException("The folder '"+ folderName + "' does not exist");
		} else if (!folder.isDirectory()) {
			throw new NotAFolderException("'" + folderName + "' is not a folder"); 
		} else {
			File file = new File(folder, fileName);
			
			/**
			 * Sequence of checks not mixed with business logic code
			 */
			if (!forWriting && !file.exists()) {
				throw new FileNotFoundException(fileName);
			}
			// the second forWriting is not actually required
			if ((!forWriting || forWriting && file.exists()) && !file.isFile()) {
				throw new NotAFileException(folderName, fileName);
			} 
			
			if (forWriting && file.exists() && !file.canWrite()) {
				throw new FileNotWriteableException(fileName);
			} 
			
			return file;
		}	
	}

	public void createBlankFile(String folderName, String fileName, String defaultContent) {
		File file = null;
		PrintStream ps = null;
		file = getFileInDirectory(folderName, fileName, true);
		
		file.createNewFile();
		ps = new PrintStream(file);
		ps.println(defaultContent);
		ps.close();
	}
}
