package es.ua.dlsi.prog3.exceptions.simple;

import java.util.*;

/**
 * Para arreglar el problema en UD3_5_LanzarExcepcion_2
 * vamos a lanzar diferentes excepciones para diferenciar 
 * las distintas condiciones de error que se pueden producir en f()
 * 
 * @author pierre
 *
 */

public class  UD3_6_LanzarExcepcionesDiferenciadas {
	/** 
	 * Función que detecta un error y lanza excepciones. 
	 * No trata los errores. Eso lo hace el código cliente, que sabe en qué contexto
	 * se ha llamado a f().
	 * 
	 * Fíjate en la cláusula 'throws' tras la lista de argumentos. 
	 * Es necesario indicarla cuando lanzamos excepciones declaradas (checked exceptions)
	 *
	 * Esta función solo admite números positivos entre cero y 100 ...
	 * @param a un número positivo entre 0 y 100
	 * @return a+1 
	 * @throws ArithmeticException si a no es positivo
	 * @throws Exception si a es mayor que 100
	 */ 
    static int f(int a)   {
        
    	if (a<0) 
        	throw new ArithmeticException("Número negativo");
        else if (a>100)
        	throw new RuntimeException("Número mayor que 100");
        
        return a+1;
    }
        
    public static void main(String args[]) {
        int x = 10000;
        
      
        // Múltiples catch, uno para cada tipo de excepción
        try {
        	int r = f(x); // o f(-x);
			System.out.println("x = "+x+" f(x)= "+r);
        }
        catch (ArithmeticException ex) {
        	System.err.println(ex.getMessage());
        } 
        catch (Exception ex) {
	        System.err.println(ex.getMessage());
	        System.exit(-1);
        }

        // Exception es más general que ArithmeticException
        // Los catch de excepciones más generales deben ponerse tras
        // los catch más específicos.
        //
        // Invertir el orden de los catch anteriores daría error de compilación, pues nunca
        // se ejecutaría el catch de ArithmeticException, ya que el de Exception capturaría ambos tipos
        // de excepción.
        
        System.out.println("FIN");
    }
}