package es.ua.dlsi.prog3.exceptions.simple;

import java.util.*;

/**
 * Para arreglar el problema en UD3_5_LanzarExcepcion_2
 * vamos a definir nuestras propias excepciones para diferenciar 
 * las distintas condiciones de error que se pueden producir en f()
 * 
 * @author pierre
 *
 */

/**
 * Excepción que se lanza cuando no se esperaba un número negativo
 * @author pierre
 *
 */
class ExcepcionNumeroNegativo extends Exception {
	int negativo;
	
	// Pasamos un mensaje descriptivo al constructor
	public ExcepcionNumeroNegativo(int n) {
		negativo = n;
	}
	
	// Recuperamos el mensaje que se le pasó al constructor cuando
	// se lanzó la excepción
	public String getMessage() {
		return "Numero negativo : "+negativo;
	}
}

/**
 * Excepción que se lanza cuando no se esperaba un número tan grande
 * @author pierre
 *
 */
class ExcepcionNumeroMuyGrande extends Exception {
	String mensaje;
	
	public ExcepcionNumeroMuyGrande(String mesg) {
		mensaje = mesg;
	}
	
	public String getMessage() {
		return mensaje;
	}
}

public class  UD3_7_LanzarExcepcionesDeUsuario {
	/** 
	 * Función que detecta un error y lanza excepciones. 
	 * No trata los errores. Eso lo hace el código cliente, que sabe en qué contexto
	 * se ha llamado a f().
	 * 
	 * Fíjate en la cláusula 'throws' tras la lista de argumentos. 
	 * Es necesario indicarla cuando lanzamos excepciones declaradas (checked exceptions)
	 *
	 * Esta función solo admite números positivos entre cero y 100 ...
	 * @param a un número positivo entre 0 y 100
	 * @return a+1 
	 * @throws ExcepcionNumeroNegativo si a no es positivo
	 * @throws ExcepcionNumeroMuyGrande si a es mayor que 100
	 */ 
    static int f(int a) throws ExcepcionNumeroNegativo, ExcepcionNumeroMuyGrande {
        if (a<0) 
        		throw new ExcepcionNumeroNegativo(a);
        else if (a>100)
        		throw new ExcepcionNumeroMuyGrande("Número mayor que 100");
        	return a+1;
    }
    
    public static void main(String args[]) {
        int x = 10;
        
        // Múltiples catch, uno para cada tipo de excepción
        try {
        		int r = f(x); // o f(-x);
			System.out.println("x = "+x+" f(x)= "+r);
        } catch (ExcepcionNumeroNegativo ex) {
	        System.err.println(ex.getMessage());
        } catch (ExcepcionNumeroMuyGrande ex) {
	        System.err.println(ex.getMessage());
        }
        
        // ¡Catch múltiple!
        // En el try/catch anterior, hacemos lo mismo en los dos catch
        // Podemos juntarlos en uno
        try {
        		int r = f(x); // o f(-x);
			System.out.println("x = "+x+" f(x)= "+r);
        } catch (ExcepcionNumeroNegativo | ExcepcionNumeroMuyGrande ex) {
	        if (ex instanceof ExcepcionNumeroMuyGrande) {
        	System.err.println(ex.getMessage());
        } 
    }
    }
}