package es.ua.dlsi.prog3.frameworks.carreras;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Simulación de un circuito de carreras
 * @author pierre
 *
 */
public class Circuito {

	/**
	 * longitud del circuito en km.
	 */
	private int longitudkm;
	/**
	 * Aforo del circuito
	 */
	private int aforo;
	/**
	 * Nombre del circuito
	 */
	private String nombre;
	/**
	 * Lista de corredores participantes en la carrera
	 */
	private List<ICorredor> lv;
	/**
	 * Coche auxiliar (safety car)
	 */
	private ICocheAuxiliar sc;

	/**
	 * Crea un circuito de nombre 'n', sin corredores ni coche auxiliar
	 * @param n nombre del circuito
	 */
	public Circuito (String n) {
		sc = null;
		nombre = n;
		lv = new ArrayList<ICorredor>();
	}

	/**
	 * devuelve el número de participantes en la carrera
	 * @return número de participantes en la carrera
	 */
	public int get_nvehiculos () 
	{ return lv.size(); }
	
	/**
	 * Longitud del circuito en km.
	 * @return longitud del circuito
	 */
	public int get_longitudkm ()
	 { return longitudkm; }

	/**
	 * Aforo del circuito
	 * @return aforo
	 */
	public int get_aforo () 
	{ return aforo; }

	/**
	 * Añade un participamte a la carrera
	 * @param c vehículo participante
	 */
	public void add_vehiculo (ICorredor c) 
	{ lv.add(c); }

	/**
	 * Asigna un coche auxiliar a la carrera
	 * @param ca el coche auxiliar
	 */
	public void add_safetycar (ICocheAuxiliar ca)
	 { sc = ca; }

	/** 
	 * Simulación del proceso de carrera
	 * @param nv número de vueltas
	 */
	public void simular_carrera (int nv) {
    System.out.println("Bienvenidos al circuito de " + nombre);
    if (sc != null) {
      System.out.println("Comienza la carrera:\n");
      while (nv >0) {
	System.out.println("["); 
         for (ICorredor v : lv) {
           if (!sc.en_pista ())
	    v.dar_vuelta ();
	  else
	    System.out.println("SafetyCar en pista"); 
	}
	System.out.println("]\n"); 
	nv--;

	if((new Random()).nextInt(100) > 50) {
	  sc.toggle ();
	}
      }
    } else 
      System.out.println("¡No hay safety-car!");
  }
}
