package es.ua.dlsi.prog3.frameworks.jcf;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Scanner;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

/**
 * Ejemplos de uso del Java Collection Framework
 * @author pierre
 *
 */
public class JCFUseCases {

	/**
	 * Constructor
	 */
	public JCFUseCases() {}
	
	/**
	 * Rellena un Collection con algunos enteros
	 * @param col Collection a rellenar
	 */
	public static void rellenaColeccion(Collection<Integer> col) {
		Random random = new Random(System.nanoTime());
		System.out.print("Insertando 10 elementos en un "+col.getClass().getName()+": ");
		int x=0;
		for (int i=0; i<10; i++) {
			x = random.nextInt(5);
			System.out.print(x+", ");
			col.add(x);
		}
		System.out.println();
	}

	/**
	 * Rellena un mapa con algunos pares <entero, cadena>
	 * @param map el mapa a rellenar
	 */
	public static void rellenaMapa(Map<Integer,String> map) {
		String input = "Bienvenido al espacio de prácticas de la asignatura Programación 3";
		Scanner sc = new Scanner(input);
		Random random = new Random(System.nanoTime());
	
		System.out.print("Insertando varios pares <int,String> en un "+map.getClass().getName()+": ");

		int x = 0;
		String s=null;
		while (sc.hasNext()) {
			x = random.nextInt(5);
			s = sc.next();
			System.out.print("<"+x+","+s+">, ");
			map.put(x, s);
		}
		System.out.println();

	}

	/**
	 * Programa principal
	 * Crea distintas colecciones y mapas de objetos del JCF:
	 * Listas, conjuntos, mapas, mapas ordenados y conjuntos ordenados,
	 * con diferentes implementaciones (ArrayList, HashMap, HashSet,...).
	 * Los rellena con varios elementos, imprime información sobre ellos
	 * y les aplica algunos algoritmos de la clase Collections (ordenar, desordenar, etc)
	 * @param args no se usa
	 */
	public static final void main(String args[]) {
		List<Integer> list;
		Map<Integer, String> map;
		Set<Integer> set;
		SortedMap<Integer, String> smap;
		SortedSet<Integer> sset;
		
		list = new ArrayList<>();
		map = new HashMap<>();
		set = new HashSet<>();

		smap = new TreeMap<>();
		sset = new TreeSet<>();
		

		List<Collection<Integer>> cols = new ArrayList<>();
		cols.add(list);
		cols.add(set);
		cols.add(sset);

		List<Map<Integer,String>> maps = new ArrayList<>();
		maps.add(map);
		maps.add(smap);
		
		for (Collection<Integer> col : cols) {
		 rellenaColeccion(col);			
		 System.out.println("La colección es un "+col.getClass().getName());
		 System.out.println("Size: "+col.size());
		 System.out.println("Vacia?: "+col.isEmpty());
		 System.out.println("Colección: "+col);
		 System.out.println("Contiene el 4: "+col.contains(new Integer(4)));
		 System.out.println("Quitar el 4: "+col.remove(new Integer(4)));
		 System.out.println("Colección: "+col);
		 System.out.println("Las dos colecciones contiene los mismos objetos: "+col.containsAll(list)+"\n");
		}

		for (Map<Integer,String> m : maps) {			
			 rellenaMapa(m);			
			 System.out.println("El mapa es un "+m.getClass().getName());
			 System.out.println("Size: "+m.size());
			 System.out.println("Vacio?: "+m.isEmpty());
			 System.out.println("Mapa: "+m);
			 System.out.println("Contiene el <4,?>: "+m.containsKey(4));
			 System.out.println("Quitar el <4,?>: "+m.remove(4));
			 System.out.println("Mapa: "+m+"\n");	
		}

		for (Collection<Integer> col : cols) {
		  System.out.println("Elementos de la colección "+col.getClass().getName());
			Iterator<Integer> it = col.iterator();
			while (it.hasNext()) {
			  System.out.print(it.next()+", ");
			}
			System.out.println();
		  }
		
		System.out.println("List: "+list);
		Collections.reverse(list);
		System.out.println("Reversed: "+list);
		Collections.sort(list);
		System.out.println("Sorted: "+list);
		Collections.shuffle(list);
		System.out.println("Shuffled: "+list);
		
		Comparator<Integer> c = 				
				new Comparator<Integer>() {
			// Primero impares, luego pares
			@Override
			public int compare(Integer o1, Integer o2) {
				int i = o1;
				int j = o2;
				if (i % 2 != 0 && j % 2 == 0) // i impar, j par
					return -1;
				if (i % 2 == 0 && j % 2 != 0) // i par, j impar
					return 1;
				return (i-j);
			}
		};
		
		Collections.sort(list, c);
		System .out.println("Sorted, pero raro: "+list);
	
	}

}
