package es.ua.dlsi.prog3.frameworks.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Demostración de conexión a una base de datos SQLite
 * mediante JDBC
 * @author pierre, drizo
 *
 */
public class SQLiteJDBCExample {

	/**
	 * Constructor
	 */
	public SQLiteJDBCExample() {}

	/**
	 * Programa principal
	 * @param args no se usa
	 */
	public static void main(String[] args) {
		try {
			// cargamos driver
			Class.forName("org.sqlite.JDBC"); 
			
			// Cadena de conexión 
			String dbURL = "jdbc:sqlite:mibbdd.db";
			// Conectamos
			Connection con = DriverManager.getConnection(dbURL, "milogin", "mipassword");	
			
			// CONSULTAMOS, INSERTAMOS, BORRAMOS usando 'con' 	
			// INSERCION
			Statement stmt = con.createStatement(); // usamos java.sql.Statement y no la de MySQL
			String sqlInsercion = "INSERT INTO  equipo (nombre, abreviatura) values ('Valencia', 'VAL'), ('Levante',  'LEV')";
			int filasInsertadas = stmt.executeUpdate(sqlInsercion);
			System.out.println("Se han insertado " + filasInsertadas + " registros");		

			// CONSULTA
			String sqlConsulta = "SELECT abreviatura, nombre from equipo order by abreviatura";
			ResultSet rstEquipos = stmt.executeQuery(sqlConsulta);
			while( rstEquipos.next() ) {
			   String abrv = rstEquipos.getString("abreviatura");
			   String nombreCompleto = rstEquipos.getString("nombre");
			   System.out.println(abrv + "\t" + nombreCompleto);
			}
			
			// PreparedStatement
			PreparedStatement preparedSQL = 
					con.prepareStatement("select * from equipo where abreviatura = ?"); 
			for (String abreviatura : new String[] { "VAL", "LEV" }) {
				preparedSQL.setString(1, abreviatura);
				rstEquipos = preparedSQL.executeQuery();
				while( rstEquipos.next() ) {
					String abrv = rstEquipos.getString("abreviatura");
					String nombreCompleto = rstEquipos.getString("nombre");
					System.out.println(abrv + "\t" + nombreCompleto);
				}				
			}
					
			// BORRADO
			String sqlBorrado = "delete from equipo where abreviatura = 'VAL' or abreviatura = 'LEV'";
			int filasBorradas = stmt.executeUpdate(sqlBorrado);
			System.out.println("Se han borrado " + filasBorradas + " registros");		
			
			// cerramos conexión al terminar
			con.close(); 
			
		} catch (ClassNotFoundException | SQLException e) {
			e.printStackTrace();
		} 
	}

}
