package es.ua.dlsi.prog3.frameworks.opsbinarias;

import java.util.Scanner;

/**
 * Framework para implementar operaciones binarias. Se proporciona el comportamiento general de una operación
 * binaria, dado unos operandos de tipo IOperando y un operador de tipo Operador.
 * 
 * Esta clase Aplicacion realiza una operación binaria en la salida estándar, solicitando los operadores y la operación a
 * realizar al usuario, para luego imprimir el resultado de la operación.
 * 
 * El usuario del framework debe especificar 
 *  - cómo se lee un operando de la entrada (IOperando)
 *  - cómo se representa (en forma de cadena) (IOperando)
 *  - cómo se realiza la operacíon (Operando)
 * 
 * En los paquetes es.ua.dlsi.prog3.frameworks.opsenteros y es.ua.dlsi.prog3.frameworks.opscjtos
 * hay sendos ejemplos de uso de este framework para operar con enteros y con conjuntos, respectivamente.
 * 
 * @author drizo
 **/
public abstract class Aplicacion<TipoOperando extends IOperando, TipoOperador extends Operador<TipoOperando>> {
	
	/**
	 * Método factoría para crear un operador
	 * @return un operador
	 */
	public abstract TipoOperador creaOperador();

	/**
	 * Método factoría para crear un operando
	 * @return un operando
	 */
	public abstract TipoOperando creaOperando();

	/**
	 * Constructor. Lanza el proceso run(). 
	 */
	public Aplicacion() {
		run();
	}
	
	/**
	 * Solicita dos operandos y un operador desde la entrada estándar e imprime en la salida estándar
	 * el resultado de la operación.
	 */
	private void run() {
		Scanner scanner = new Scanner(System.in);
		try {

			System.out.println("Introduce el primer operando:");		
			TipoOperando a = creaOperando();
			a.lee(scanner.nextLine());
			
			TipoOperador op = creaOperador();
			System.out.println("Introduce la operación:");
			op.lee(scanner.nextLine());
			
			System.out.println("Introduce el segundo operador:");
			TipoOperando b = creaOperando();
			b.lee(scanner.nextLine());
			
			TipoOperando resultado = op.calcula(a, b);
			System.out.println("El resultado de la operación ");
			System.out.print(a.toString());
			System.out.print(op.toString());
			System.out.print(b.toString());
			System.out.print("=");
			System.out.println(resultado.toString());
		} catch (ExcepcionFramworkOps e) {
			System.err.println("Error: " + e.getMessage());
		} finally {
			scanner.close();
		}
	}
}
