package es.ua.dlsi.prog3.frameworks.opscjtos;


import java.util.Set;
import java.util.TreeSet;

import es.ua.dlsi.prog3.frameworks.opsbinarias.ExcepcionFramworkOps;
import es.ua.dlsi.prog3.frameworks.opsbinarias.Operador;

/**
 * Operador para conjuntos de cadenas. Soporta la unión ('U') y la intersección ('^')
 * de conjuntos.
 * @author drizo
 **/
public class OperadorCjtoCads extends Operador<OperandoCjtoCads> {
	
	
	/**
	 * Calcula la unión o intersección de cadenas, dependiendo del operador creado
	 * @throws ExcepcionFramworkOps si el operando es desconocido.
	 */
	@Override
	public OperandoCjtoCads calcula(OperandoCjtoCads a, OperandoCjtoCads b)
			throws ExcepcionFramworkOps {
		if (toString().equals("U")) {
			return union(a,b);
		} else if (toString().equals("^")) {
			return interseccion(a,b);
		} else {
			throw new ExcepcionFramworkOps("Operando inválido: " + toString());
		}
	}
	
	/**
	 * Calcula la intersección de dos conjuntos de cadenas
	 * @param a primer conjuntp
	 * @param b segundo conjunto
	 * @return la intersección de los conjuntos 'a' y 'b'
	 */
	private OperandoCjtoCads interseccion(OperandoCjtoCads a,
			OperandoCjtoCads b) {
		Set<String> res = new TreeSet<String>(a.getCjto());
		res.retainAll(b.getCjto());
		return new OperandoCjtoCads(res);
	}

	/**
	 * Calcula la unión de dos conjuntos de cadenas
	 * @param a primer conjuntp
	 * @param b segundo conjunto
	 * @return la unión de los conjuntos 'a' y 'b'
	 */
	private OperandoCjtoCads union(OperandoCjtoCads a,
			OperandoCjtoCads b) {
		Set<String> res = new TreeSet<String>(a.getCjto());
		res.addAll(b.getCjto());
		return new OperandoCjtoCads(res);
	}
}
