/**
 * 
 */
package es.ua.dlsi.prog3.generics;

/**
 * ArrayPila como clase genérica que implementa un interfaz genérico (Pila)
 * (comparar con es.ua.dlsi.prog3.inheritance.secure.ArrayPila) 
 * @author pierre
 *
 */
public class ArrayPila<T> implements Pila<T> {

	/**
	 * Tamaño máximo para este tipo de pila
	 */
	public static final int MAX_PILA=10;
	/**
	 * La pila, implementada como un array.
	 */
	private T[] pila;
	/**
	 * índice del último elemento apilado
	 */
	private int tope;
	
	/** 
	 * Constructor
	 */
	public ArrayPila() {
		pila = (T[]) new Object[MAX_PILA]; // no podemos hacer 'new T[MAX_PILA]'
		tope=-1;
	}

	/**
	 * {inheritDoc}
	 */
	@Override
	public void apila(T o) {
			if (tope<MAX_PILA)
				pila[++tope] = o;
			else
				throw new IndexOutOfBoundsException("Pila llena");
	}

	/**
	 * {inheritDoc}
	 */
	@Override
	public T desapila() {
		if (tope!= -1)
			return pila[tope--];
		else return null;
	}

	/**
	 * {inheritDoc}
	 */
	@Override
	public int size() { return tope+1; }
	
	/**
	 * {inheritDoc}
	 */
	@Override
	public T tope() {
		// Sin hacer copia defensiva es fácil:
		if (tope!=-1) return pila[tope];
		else return null;

		// Con COPIA DEFENSIVA de un objeto que no sabemos de qué clase es (bueno, sí, es un Object)
		/* Si la clase (en tiempo de ejecución) del objeto no tiene constructor de copia
		 * obtendremos una 'NoSuchMethodException'
		Object ret=null;
		if (tope!=-1) {
			Class<?> clase = pila[tope].getClass();
			Class<?>[] paramTypes = new Class[] { clase };
			Constructor<?> ctor;
			try {
				ctor = clase.getConstructor( paramTypes );
				Object[] arguments = new Object[] { pila[tope] };
				ret = ctor.newInstance(arguments);
			} catch (NoSuchMethodException | SecurityException | InstantiationException 
					| IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		return ret;
		*/
	}

}
