/**
 * Demostración de genericidad restringida/parámetros de tipo acotados
 */
package es.ua.dlsi.prog3.generics.restricted;

import java.util.ArrayList;
import java.util.List;

/**
 * Un perro que puede ladrar y alimentarse
 * @author pierre
 *
 */
class Perro {
	public boolean ladrar() { return true; }
	public void alimentar() {}
}

/*
 * Dos clases de Perro: Mastin y PastorBelga
 */
class Mastin extends Perro {}

class PastorBelga extends Perro {}

/**
 * Una Perrera para acoger varios perros
 * @author pierre
 *
 * @param <T> Algún tipo de Perro
 */
public class Perrera<T extends Perro> {
	
	  private List<T> jaula =  new ArrayList<T>();

	  /**
	   * Acoger un perro
	   * @param p el perro
	   */
	  public void acoger(T p) { jaula.add(p); }
	  
	  /**
	   * Alimentar a todos los perros de la perrera
	   */
	  public void alimentar() { 
	    for (T p : jaula) if (p.ladrar()) p.alimentar();
	  }
	  
	  /**
	   * Adoptar un perro y sacarlo de la perrera
	   * @param i el índice del perro a adoptar
	   * @return el perro adoptado
	   */
	  public T adoptar(int i) { return jaula.remove(i); }

	  /**
	   * Programa principal
	   * Crea una perrera de mastines
	   * @param args no se usa
	   */
	  public static final void main(String args[]) {
		  Perrera<Mastin> pm = new Perrera<>();
		  pm.acoger(new Mastin());
		  Perro mastin = pm.adoptar(0);
		  //pm.acoger(new PastorBelga());
		  
		  /*
		   * Piensa: ¿Qué ventaja tiene usar esta clase Perrera respecto a usar una definida simplemente como
		   *   class Perrera {
		   *     private List<Perro> jaula = ...
		   *     ...
		   *   }  ?
		   */
	  }
}