/**
 * Demostración de comodines de subtipo y comodines de supertipo
 */
package es.ua.dlsi.prog3.generics.wildcards;

import java.util.ArrayList;
import java.util.List;

class Vegetal {
	public String getVitaminas() { return "Muchas vitaminas"; }
}
class Fruit extends Vegetal {
	public String getZumo() { return "Zumo de Fruta" ; }
}
class Apple extends Fruit {
	public String getVitaminas() { return "Vitamina B12"; }
	public String getZumo() { return "Zumo de Manzana" ; }	
}
class Orange extends Fruit {
	public String getVitaminas() { return "Vitamina C++"; }
	public String getZumo() { return "Zumo de Naranja" ; }
}

public class GenericWildcards {

	/**
	 * Constructor
	 */
	public GenericWildcards() {
		List<Fruit> flist = new ArrayList<Fruit>();
		// Si nos dejaran hacer esto:
		  flist = new ArrayList<Apple>();
		// Entonces podríamos guardar naranjas en una lista de manzanas:
		  flist.add(new Orange());
		
	}
	
	/**
	  Comodín de subtipo
	  'frutas' ha sido instanciado con Fruit o subclases de Fruit
	  Sólo puedo 'leer' del List, no puedo insertar
	  Sé que lo que obtengo son Fruit seguro.
	 * 
	 * @param frutas una lista de frutas
	 */
	public static void procesarFrutas(List<? extends Fruit> frutas){
		for(Fruit a : frutas){
			      System.out.println(a.getZumo());
			      
		}
	}
	
	/**
		  Comodín de supertipo
		  'frutas' ha sido instanciado con Fruit o superclases de Fruit
		  Por tanto, sé que puedo insertar cualquier cosa que sea al menos un Fruit
		  Sin embargo, no sé que puedo obtener. Como mucho un Object...
	 * 
	 * @param frutas
	 */
		public static void insertarFrutas(List<? super Fruit> frutas){
			frutas.add(new Apple()); // OK
			frutas.add(new Fruit()); // OK
		
			// Fruit f = frutas.get(0); // Quizás no!!
			// Sólo puedo asumir que son Object
			for( Object a : frutas){
				      System.out.println(a.toString());
			}
			
		}

		public static final void main(String args[]) {
			procesarFrutas(new ArrayList<Fruit>());
			procesarFrutas(new ArrayList<Apple>());
			procesarFrutas(new ArrayList<Orange>());

			insertarFrutas(new ArrayList<Fruit>());
			insertarFrutas(new ArrayList<Vegetal>());
			insertarFrutas(new ArrayList<Object>());
			// insertarFrutas(new ArrayList<Apple>());


			// Una lista de frutas
			List<Fruit> frutas = new ArrayList<Fruit>();
			frutas.add(new Apple());
			frutas.add(new Orange());
			frutas.add(new Fruit());

			// 'l' es una lista instanciada con cualquier tipo (en tiempo de ejecución)
			List<?> l = new ArrayList<Fruit>();

			// En tiempo de compilación, no sabemos que tipo de objetos contendrá la lista 'l', no podemos
			// insertar en ella nada, ¡ni siquiera Object!
			l.add(new Fruit());
			l.add((Object) new Fruit());
			// Sin embargo, si podemos consultar la lista. Sabemos que contiene, al menos, Objects:
			Object f = l.get(0);

		}
}
