package es.ua.dlsi.prog3.inheritance.destruction.dispose;
/**
 * (tomado de 'Piensa en Java,4ª ed.', Bruce Eckl)
 * Demostración de otra forma de ejecutar código cuando un objeto es destruído
 * por el recolector de basura de la máquina virtual de Java
 * 
 * Se usa un método definido por el usuario, dispose(), en una jerarquía de clases, similar 
 * a un destructor en C++. A diferencia de los destructores en C++, estos métodos deben
 * invocarse explícitamente. Fíjate que el orden de llamadas entre los métodos dispose() y de ejecución
 * es justo el contrario del orden de llamada y ejecución de los constructores.
 * 
 * @author pierre
 *
 */
class Shape {
	Shape(int i) { print("Shape ctor"); }
	
	void dispose() { print("Shape dispose"); }
	
	void print(String s) {
		System.out.println(s);
	}
}

class Circle extends Shape {
	Circle(int i) {
		super(i);
		print("Drawing Circle");
	}
	
	void dispose() {
		print("Erasing Circle");
		// la llamada a la superclase es la última instrucción, es decir, se realiza DESPUES de la ejecución de este método
		super.dispose();
	}
}

class Triangle extends Shape {
	Triangle(int i) {
		super(i);
		print("Drawing Triangle");
	}
	void dispose() {
		print("Erasing Triangle");
		super.dispose();
	}
}

public class CADSystem extends Shape {
	private Circle c;
	private Triangle t;

	public CADSystem(int i) {
		super(i + 1);
		c = new Circle(1);
		t = new Triangle(1);
		print("Combined constructor");
	}
	public void dispose() {
		print("CADSystem.dispose()");
		// The order of cleanup is the reverse
		// of the order of initialization:
		t.dispose();
		c.dispose();
		super.dispose();
	}
	public static void main(String[] args) {
		CADSystem x = new CADSystem(47);
		try {
			// Code and exception handling...
		} finally {
			x.dispose();
		}
	}
}
