/**
 * Demostración de refactorización, basado en el ejemplo del capítulo 1 de 
 * 'Refactoring. Improving the Design of Existing Code', de Martin Fowler.
 */
package es.ua.dlsi.prog3.refactoring.videoclub;

import es.ua.dlsi.prog3.refactoring.videoclub.original.Alquiler;
import es.ua.dlsi.prog3.refactoring.videoclub.original.Cliente;
import es.ua.dlsi.prog3.refactoring.videoclub.original.Pelicula;
import es.ua.dlsi.prog3.refactoring.videoclub.refactored.PeliculaEstreno;
import es.ua.dlsi.prog3.refactoring.videoclub.refactored.PeliculaInfantil;
import es.ua.dlsi.prog3.refactoring.videoclub.refactored.PeliculaNormal;

/**
 * Clase del programa principal.
 * Permite ejecutar la versión original del código (sin refactorizar)
 * y la refactorizada.
 * 
 */
public class Videoclub {


	public static final void main(String args[]) {
		System.out.println("Implementacion ORIGINAL (sin refactorizar): ");
		runOriginal();	
		// Quita estos comentarios pra ejecutar la versión refactorizada.
		// Debería dar el mismo resultado que la original.
		//System.out.println("\nImplementacion REFACTORIZADA: ");
		//runRefactored();
	}

	private static void runOriginal() {

		Pelicula p1=new Pelicula("STAR WARS XXV", Pelicula.ESTRENO);
		Pelicula p2=new Pelicula("STAR TRECK", Pelicula.NORMAL);
		Pelicula p3=new Pelicula("SHRECK", Pelicula.INFANTIL);

		Cliente c = new Cliente("JUAN TRECKY");

		c.addAlquiler(new Alquiler(p1, 3));
		c.addAlquiler(new Alquiler(p2, 4));
		c.addAlquiler(new Alquiler(p3, 5));

		System.out.println(c.informe());
		System.out.println(c.informeHTML());
	}

	/* Cuando hayas terminado de refactorizar el código, usa esta versión de runRefactored() para ejecutarlo */
	/*
	private static void runRefactored() {
		es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Pelicula p1 =
				new PeliculaEstreno("STAR WARS XXV");
		es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Pelicula p2 =
				new PeliculaNormal("STAR TRECK");
		es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Pelicula p3 =
				new PeliculaInfantil("SHRECK");

		es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Cliente c = 
				new es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Cliente("JUAN TRECKY");

		c.addAlquiler(new es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Alquiler(p1, 3));
		c.addAlquiler(new es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Alquiler(p2, 4));
		c.addAlquiler(new es.ua.dlsi.prog3.refactoring.videoclub.original.copy.Alquiler(p3, 5));

		System.out.println(c.informe());

	}*/

	private static void runRefactored() {
		es.ua.dlsi.prog3.refactoring.videoclub.refactored.Pelicula p1 =
				new PeliculaEstreno("STAR WARS XXV");
		es.ua.dlsi.prog3.refactoring.videoclub.refactored.Pelicula p2 =
				new PeliculaNormal("STAR TRECK");
		es.ua.dlsi.prog3.refactoring.videoclub.refactored.Pelicula p3 =
				new PeliculaInfantil("SHRECK");

		es.ua.dlsi.prog3.refactoring.videoclub.refactored.Cliente c = 
				new es.ua.dlsi.prog3.refactoring.videoclub.refactored.Cliente("JUAN TRECKY");

		c.addAlquiler(new es.ua.dlsi.prog3.refactoring.videoclub.refactored.Alquiler(p1, 3));
		c.addAlquiler(new es.ua.dlsi.prog3.refactoring.videoclub.refactored.Alquiler(p2, 4));
		c.addAlquiler(new es.ua.dlsi.prog3.refactoring.videoclub.refactored.Alquiler(p3, 5));

		System.out.println(c.informe());
	}
}

