/**
 * Demostración de refactorización, basado en el ejemplo del capítulo 1 de 
 * 'Refactoring. Improving the Design of Existing Code', de Martin Fowler.
 * 
 * Código refactorizado. Así debería quedar, más o menos, tu refactorización.
 */
package es.ua.dlsi.prog3.refactoring.videoclub.refactored;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class Cliente {

	private String nombre;
	private List<Alquiler> _rentals;
	
	public Cliente(String nom) {
		nombre = nom;
		_rentals = new ArrayList<>();
	}

	public String getNombre() { return nombre; }
	
	public void addAlquiler(Alquiler a) {
		_rentals.add(a);
	}
	
	public String informe() {
		double totalAmount = 0;
		int frequentRenterPoints = 0;
		Iterator<Alquiler> rentals = _rentals.iterator();
		String result = "Rental Record for " + getNombre() + "\n";
			
		while (rentals.hasNext()) {
			Alquiler each = rentals.next();
			
			double thisAmount = each.getCargoAlquiler();
			frequentRenterPoints += each.getFrequentRenterPoints();
			totalAmount += thisAmount;
					
			// show figures for this rental
			result += "\t" + each.getPelicula().getTitulo() 
			+ "\t" + String.valueOf(thisAmount) + "\n";
			
		  } // END WHILE
				
			// add footer lines
			result += "Amount owed is " + String.valueOf(totalAmount) + "\n";
			result += "You earned " + String.valueOf(frequentRenterPoints) 
			+ " frequent renter points";
				
			return result;
	}

	
}
