package es.ua.dlsi.prog3.refactoring.videoclub.original;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

public class ClienteTest {

	Pelicula p1, p2, p3;
	Alquiler alq1;
	Cliente c1, c2;
	
	@Before
	public void setUp() throws Exception {
		p1 = new Pelicula("SHREK", Pelicula.INFANTIL);
		p2 = new Pelicula("STAR WARS XXX", Pelicula.ESTRENO);
		p3 = new Pelicula("LOS PUENTES DE MADISON", Pelicula.NORMAL);
		
		c1 = new Cliente("JUAN");
		c2 = new Cliente("PEPE");
	}

	@Test
	public final void testCliente() {
		assertEquals("JUAN",c1.getNombre() );
	}


	@Test
	public final void testInforme1() {
		
		String expected = "Rental Record for JUAN\nAmount owed is 0.0\nYou earned 0 frequent renter points";
		assertEquals(expected,c1.informe());
	}

	@Test
	public final void testAddAlquiler() {
		c1.addAlquiler(new Alquiler(p1, 3));
		String expected = "Rental Record for JUAN\n\tSHREK\t1.5\nAmount owed is 1.5\nYou earned 1 frequent renter points";
		assertEquals(expected,c1.informe());		
	}

	// MARTIN FOWLER tests
	@Test
	public void testCliente2() {
		Cliente c = new Cliente("David");
		assertNotNull(c);	
	}

	@Test
	public void testAddAlquiler2() {
		Cliente Cliente2 = new Cliente("Sallie");
		Pelicula Pelicula1 = new Pelicula("Gone with the Wind", Pelicula.NORMAL);
		Alquiler Alquiler1 = new Alquiler(Pelicula1, 3); // 3 day Alquiler
		Cliente2.addAlquiler(Alquiler1);
	}

	@Test
	public void testGetName() {
		Cliente c = new Cliente("David");
		assertEquals("David", c.getNombre());
	}

	@Test
	public void testStatementForNORMALPelicula() {
		Cliente Cliente2 = new Cliente("Sallie");
		Pelicula Pelicula1 = new Pelicula("Gone with the Wind", Pelicula.NORMAL);
		Alquiler Alquiler1 = new Alquiler(Pelicula1, 3); // 3 day Alquiler
		Cliente2.addAlquiler(Alquiler1);
		String expected = "Rental Record for Sallie\n" +
							"\tGone with the Wind\t3.5\n" +
							"Amount owed is 3.5\n" +
							"You earned 1 frequent renter points";
		String statement = Cliente2.informe();
		assertEquals(expected, statement);
	}
	
	@Test
	public void testStatementForNewReleasePelicula() {
		Cliente Cliente2 = new Cliente("Sallie");
		Pelicula Pelicula1 = new Pelicula("Star Wars", Pelicula.ESTRENO);
		Alquiler Alquiler1 = new Alquiler(Pelicula1, 3); // 3 day Alquiler
		Cliente2.addAlquiler(Alquiler1);
		String expected = "Rental Record for Sallie\n" +
							"\tStar Wars\t9.0\n" +
							"Amount owed is 9.0\n" +
							"You earned 2 frequent renter points";
		String statement = Cliente2.informe();
		assertEquals(expected, statement);
	}
	
	@Test
	public void testStatementForINFANTILPelicula() {
		Cliente Cliente2 = new Cliente("Sallie");
		Pelicula Pelicula1 = new Pelicula("Madagascar", Pelicula.INFANTIL);
		Alquiler Alquiler1 = new Alquiler(Pelicula1, 3); // 3 day Alquiler
		Cliente2.addAlquiler(Alquiler1);
		String expected = "Rental Record for Sallie\n" +
							"\tMadagascar\t1.5\n" +
							"Amount owed is 1.5\n" +
							"You earned 1 frequent renter points";
		String statement = Cliente2.informe();
		assertEquals(expected, statement);
	}
	
	@Test
	public void testStatementForManyPeliculas() {
		Cliente Cliente1 = new Cliente("David");
		Pelicula Pelicula1 = new Pelicula("Madagascar", Pelicula.INFANTIL);
		Alquiler Alquiler1 = new Alquiler(Pelicula1, 6); // 6 day Alquiler
		Pelicula Pelicula2 = new Pelicula("Star Wars", Pelicula.ESTRENO);
		Alquiler Alquiler2 = new Alquiler(Pelicula2, 2); // 2 day Alquiler
		Pelicula Pelicula3 = new Pelicula("Gone with the Wind", Pelicula.NORMAL);
		Alquiler Alquiler3 = new Alquiler(Pelicula3, 8); // 8 day Alquiler
		Cliente1.addAlquiler(Alquiler1);
		Cliente1.addAlquiler(Alquiler2);
		Cliente1.addAlquiler(Alquiler3);
		String expected = "Rental Record for David\n" +
							"\tMadagascar\t6.0\n" +
							"\tStar Wars\t6.0\n" +
							"\tGone with the Wind\t11.0\n" +
							"Amount owed is 23.0\n" +
							"You earned 4 frequent renter points";
		String statement = Cliente1.informe();
		assertEquals(expected, statement);
	}
	

}
